import unittest
from zope.testing import doctest
from zope.component import testing
from Testing.ZopeTestCase import FunctionalDocFileSuite
import Products.PloneTestCase.layer

from Products.PloneTestCase import PloneTestCase
PloneTestCase.setupPloneSite()

def test_suite():
    flags = doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS

    unitsuite = unittest.TestSuite((
        doctest.DocFileSuite('subtyping.txt',
                             setUp=testing.setUp,
                             tearDown=testing.tearDown,
                             optionflags=flags),
        ))

    fsuite = unittest.TestSuite((
        FunctionalDocFileSuite('browser.txt',
                               package='p4a.subtyper',
                               optionflags=flags),
        ))
    fsuite.layer = Products.PloneTestCase.layer.PloneSite

    return unittest.TestSuite((unitsuite, fsuite))



from zope import interface
from zope.app.publisher.browser.menu import BrowserSubMenuItem
from plone.app.contentmenu.interfaces import IActionsSubMenuItem

class SubtypesSubMenuItem(BrowserSubMenuItem):
    interface.implements(IActionsSubMenuItem)

    title = u'Sub-types'
    description = u''
    submenuId = u'subtypes'
    order = 9
    extra = {'id': 'subtypes'}

    @property
    def action(self):
        return self.context.absolute_url()+ '/subtypes'

    def available(self):
        return True




def dotted_name(obj):
    return obj.__module__ + '.' + obj.__name__

def name_to_class(name):
    pieces = name.split('.')
    ifacename = pieces[-1]
    pm = '.'.join(pieces[:-1])

    m = __import__(pm, globals(), locals(), pm)
    return getattr(m, ifacename)




import logging
import zope.component
import zope.interface
from zope.app.publisher.interfaces.browser import IBrowserMenu
from zope.app.publisher.browser.menu import BrowserMenu
try:
    from zope.component.interface import interfaceToName
except ImportError, err:
    from zope.app.component.interface import interfaceToName
from p4a.subtyper import interfaces
from p4a.subtyper import utils

logger = logging.getLogger('p4a.subtyper.menu')

class SubtypesMenu(BrowserMenu):
    """A menu with items representing all possible subtypes for the current
    context.
    """

    zope.interface.implements(IBrowserMenu)

    def _get_menus(self, object, request): 
        subtyper = zope.component.getUtility(interfaces.ISubtyper)
        existing = subtyper.existing_type(object)

        result = []
        for subtype in subtyper.possible_types(object):
            descriptor = subtype.descriptor

            selected = existing is not None and subtype.name == existing.name

            d = {'title': descriptor.title,
                 'description': descriptor.description or u'',
                 'action': '%s/@@subtyper/change_type?subtype=%s' % \
                     (object.absolute_url(), subtype.name),
                 'selected': selected,
                 'icon': '',
                 'extra': {'id': descriptor.type_interface.__name__,
                           'separator': None},
                 'submenu': None,
                 'subtype': subtype }
            result.append(d)

        return result

    def getMenuItems(self, object, request):
        try:
            return self._get_menus(object, request)
        except Exception, e:
            # it can be very difficult to troubleshoot errors here
            # because sometimes it bubbles up as AttributeError's which
            # the zope2 publisher handles in a very bizarre manner

            logger.exception(e)
            raise



import Acquisition
import zope.interface
import zope.component
from p4a.subtyper import interfaces
import Products.Archetypes.interfaces

class PossibleDescriptors(object):
    zope.interface.implements(interfaces.IPossibleDescriptors)

    def __init__(self, possible=[], comment=None):
        self._possible = possible
        self._comment = comment

    @property
    def possible(self):
        return self._possible

    def __str__(self):
        return '<PossibleDescriptors comment=%s>' % (self._comment or '')
    __repr__ = __str__

@zope.component.adapter(Products.Archetypes.interfaces.IBaseFolder)
@zope.interface.implementer(interfaces.IPossibleDescriptors)
def folderish_possible_descriptors(context):
    portal_type = getattr(Acquisition.aq_inner(context), 'portal_type', None)
    if portal_type is None:
        return PossibleDescriptors()

    possible = zope.component.getUtilitiesFor \
               (interfaces.IPortalTypedFolderishDescriptor)
    return PossibleDescriptors([(n, c) for n, c in possible
                                if c.for_portal_type == portal_type],
                               'folderish')

@zope.component.adapter(zope.interface.Interface)
@zope.interface.implementer(interfaces.IPossibleDescriptors)
def nonfolderish_possible_descriptors(context):
    portal_type = getattr(Acquisition.aq_inner(context), 'portal_type', None)
    if portal_type is None:
        return PossibleDescriptors()

    all = zope.component.getUtilitiesFor \
          (interfaces.IPortalTypedDescriptor)
    folderish = zope.component.getUtilitiesFor \
          (interfaces.IPortalTypedFolderishDescriptor)

    all = set([(n, c) for n, c in all if c.for_portal_type == portal_type])
    folderish = set([(n, c) for n, c in folderish
                     if c.for_portal_type == portal_type])

    return PossibleDescriptors(list(all.difference(folderish)),
                               'nonfolderish')



from Acquisition import aq_base
import zope.interface
import zope.component
from p4a.subtyper import interfaces
from p4a.subtyper import utils
import Products.Five.browser
from Products.statusmessages.interfaces import IStatusMessage

class ISubtyperView(zope.interface.Interface):
    def possible_types(): pass
    def has_possible_types(): pass
    def change_type(): pass

class SubtyperView(Products.Five.browser.BrowserView):
    """View for introspecting and possibly changing subtype info for the
    current context.
    """

    zope.interface.implements(ISubtyperView)

    def possible_types(self):
        subtyper = zope.component.getUtility(interfaces.ISubtyper)
        return subtyper.possible_types(self.context)

    def has_possible_types(self):
        return len(self.possible_types()) > 0

    def _redirect(self, msg):
        url = self.context.absolute_url()
        if hasattr(aq_base(self.context), 'getLayout'):
            layout = self.context.getLayout() or ''
            if layout:
                if not url.endswith('/'):
                    url += '/'
                url += layout
        IStatusMessage(self.request).addStatusMessage(msg)
        self.request.response.redirect(url)
        return ''

    def change_type(self):
        """Change the sub type of the current context.
        """

        subtyper = zope.component.getUtility(interfaces.ISubtyper)

        subtype_name = self.request.get('subtype', None)
        if subtype_name:
            existing = subtyper.existing_type(self.context)
            subtype = subtyper.get_named_type(subtype_name)
            if existing is not None and existing.name == subtype_name:
                subtyper.remove_type(self.context)
                msg = 'Removed %s subtype' % subtype.title
            else:
                subtyper.change_type(self.context, subtype_name)
                msg = 'Changed subtype to %s' % subtype.title
        else:
            msg = 'No subtype specified'

        return self._redirect(msg)



from p4a.common import site

from Products.CMFCore import utils as cmfutils
from Products.CMFCore import DirectoryView

import logging
logger = logging.getLogger('p4a.subtyper.sitesetup')

try:
    import five.localsitemanager
    HAS_FLSM = True
    logger.info('Using five.localsitemanager')
except ImportError, err:
    HAS_FLSM = False


def setup_portal(portal):
    site.ensure_site(portal)
    setup_site(portal)

def setup_site(site):
    # In 2.5, install the subtyper profile:
    mt = cmfutils.getToolByName(site, 'portal_migration')
    plone_version = mt.getInstanceVersion()
    if plone_version[0:3] == '2.5':
        # Setup only needed for Plone 3.0
        skin_tool = cmfutils.getToolByName(site, 'portal_skins')
        path = None
        for path in DirectoryView.manage_listAvailableDirectories():
            if path.endswith('p4a_subtyper'):
                break
        assert(path is None, "Subtyper skin directory not found")
        if 'p4a_subtyper' not in skin_tool.objectIds():
            DirectoryView.createDirectoryView(skin_tool, path)

        for skin_name, paths in skin_tool.getSkinPaths():
            if not 'p4a_subtyper' in paths:
                paths = paths.split(',')
                index = paths.index('plone_templates')
                paths.insert(index, 'p4a_subtyper')
                paths = ','.join(paths)
                skin_tool._getSelections()[skin_name] = paths


def unsetup_portal(portal):
    pass



import unittest
from zope.testing import doctest
from Testing.ZopeTestCase import FunctionalDocFileSuite
import Products.PloneTestCase.layer

from Products.PloneTestCase import PloneTestCase
PloneTestCase.installProduct('Plone4ArtistsSubtyper')
PloneTestCase.setupPloneSite()

def test_suite():
    flags = doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
    fsuite = unittest.TestSuite((
        FunctionalDocFileSuite('browser.txt',
                               package='p4a.subtyper.contentmenu',
                               optionflags=flags),
        ))
    fsuite.layer = Products.PloneTestCase.layer.PloneSite

    return fsuite



import Products.Five.browser
import Globals
import AccessControl.SecurityInfo

class ContentMenuView(Products.Five.browser.BrowserView):
    """A view for the content menu.
    """

    security = AccessControl.SecurityInfo.ClassSecurityInfo()

    security.declarePublic('macros')
    @property
    def macros(self):
        return self.index.macros

Globals.InitializeClass(ContentMenuView)

class TestView(Products.Five.browser.BrowserView):
    """Simple replacement of the standard zope2 test function.
    """

    def test(self, condition, trueval, falseval):
        if condition:
            return trueval
        return falseval

    def __call__(self, condition, trueval, falseval):
        return self.test(condition, trueval, falseval)
